import { ComponentDrawInfo, DateBy, GridConfig, TimelineData } from './types';
import Component from './common/Component';
import Event from './Event';
import Axis from './Axis';
import { SN } from './common/definitions';
import { ExtensionManager } from './extensions';
/**
 * @typedef {Object} EventInfo
 * @property {Date} date - a date of event fired.
 * @property {string} title - the title of event.
 *
 * @property {[string]} description - the description of event.
 * @property {[Date|'now']} endDate
 * A date of event ended if event has.
 * Set "now" to specify the event is continuous.
 * @property {[string]} endText - the description of event ended if event has.
 *
 * @property {[boolean]} folded - the event is folded or not when timeline drawn.
 * @property {[string]} foldPlaceholder - a text to replace title when event is folded.
 * */
/**
 * @property {EventInfo[]} - a event list that be drawn on timeline.
 * */
interface DrawInfo extends ComponentDrawInfo {
    events: {
        date: string;
        title: string;
        description?: string;
        endDate?: string | 'now';
        endText?: string;
        folded?: boolean;
        foldPlaceholder?: string;
    }[];
}
/**
 * The runtime info of Timeline.
 * @property {Date} startDate - the start date of timeline.
 * @property {Date} endDate - the end date of timeline.
 * @property {DateBy|null} milestoneBy - which time type is used in split the milestone.
 * @property {DateBy|null} scaleBy - which time type is used in split the scale.
 * @property {number} axisLength - the px of timeline length. It's not a whole timeline length.
 * */
export interface RuntimeInfo {
    startDate: Date;
    endDate: Date;
    milestoneBy: DateBy | null;
    scaleBy: DateBy | null;
    axisLength: number;
}
export interface ConstructInfo {
    canvas: HTMLCanvasElement;
    container: HTMLElement;
    ext?: ExtensionManager;
    grid?: GridConfig;
}
/**
 * Timeline self. There are 2 way to draw a timeline.
 * 1. draw a timeline base on event list.
 * 2. draw a timeline base an TimelineData.
 * For No.1, there are like:
 * ```js
 * const timeline = new Timeline(...);
 * timeline.event = [...];
 * await timeline.apply();
 * timeline.draw();
 * ```js
 *
 * For No.2, Ensure you had a TimelineData, and run like this for draw a timeline:
 * (The TimelineData is generate by timeline.export())
 * ```js
 * const timeline = new Timeline(...);
 * await timeline.drawFrom(timelineData);
 * ```
 * */
export default abstract class Timeline extends Component {
    constructor(props: ConstructInfo);
    name: SN;
    grid: GridConfig;
    /**
     * Generated after timeline.apply() called.
     * If you wanna to specify some keys, passed a object when timeline.apply() called.
     * @see RuntimeInfo
     * */
    runtime: RuntimeInfo;
    drawInfo: DrawInfo;
    events: Event[];
    axis: Axis;
    abstract axisConstructor: typeof Axis;
    abstract eventConstructor: typeof Event;
    createBox(): void;
    /**
     * @param {Partial<RuntimeInfo>} runtime - manually specify some runtime info.
     * */
    apply(runtime?: Partial<RuntimeInfo>): Promise<import("./common/Component").MUST_CALL_AND_RETURN_SUPER_METHOD>;
    draw(): import("./common/Component").MUST_CALL_AND_RETURN_SUPER_METHOD;
    hide(): import("./common/Component").MUST_CALL_AND_RETURN_SUPER_METHOD;
    /**
     * Draw a timeline base on a TimelineData.
     * (The TimelineData is generated by timeline.export())
     * @param (TimelineData) input
     * @return {Promise<void>>}
     * */
    import(input: TimelineData): Promise<void>;
    /**
     * Export a TimelineData for current timeline.
     * Make sure call timeline.apply() before call this method.
     * @return {TimelineData}
     * */
    export(): TimelineData;
    /**
     * Generate a pure image base timeline.
     * It's available after after draw() been called
     * */
    screenshot(): Promise<HTMLCanvasElement>;
    /**
     * Download a timeline image for user
     * It's available after after draw() been called
     * */
    downloadImage(filename?: string): Promise<void>;
    protected stretchCanvas(): void;
    protected initRuntime(runtime?: Partial<RuntimeInfo>): void;
    protected countStartData(): Date;
    protected countEndData(): Date;
    protected countMilestoneBy(): DateBy | null;
    protected countScaleBy(): DateBy | null;
    protected countAxisLength(): number;
    protected initAxis(): void;
    protected initEvents(): void;
    static defaultGrid: GridConfig;
    static is(comp: Component): comp is Timeline;
    /**
     * Mount a HTML element adding canvas and container.
     * The HTML element passed will be cleared.
     * */
    static mount(el: string | Element, themeName: string): {
        container: HTMLElement;
        canvas: HTMLCanvasElement;
    };
}
export {};
