import { Extension, ExtensionManager } from '.';
import Component from '../common/Component';
import moveto = require('moveto');
/**
 * Supported all of Breakpoint.
 * @property {string} PushScalesAndMilestones
 * When recompute the position of Scales and Milestones
 * @property {string} FixEventBody2AxisMilestone
 * When fix conflict between EventBody and AxisMilestone.
 * @property {string} FixEventAxis2EventAxis
 * @property {string} FixEventBody2EventBodyMover
 * @property {string} FixEventBody2EventBodyFloater
 * @property {string} DrawFrom - when draw a timeline by data imported.
 * @property {string} Debug
 * Set a Breakpoint for debug.
 * When debug-mode enabled, short-night will blocking at Breakpoint.Debug automatically.
 * */
export declare enum Breakpoint {
    PushScalesAndMilestones = "PushScalesAndMilestones",
    FixEventBody2AxisMilestone = "FixEventBody2AxisMilestone",
    FixEventAxis2EventAxis = "FixEventAxis2EventAxis",
    FixEventBody2EventBodyMover = "FixEventBody2EventBodyMover",
    FixEventBody2EventBodyFloater = "FixEventBody2EventBodyFloater",
    DrawFrom = "DrawFrom",
    Debug = "Debug"
}
/**
 * Config for this extension.
 * @property {Breakpoint[]} breakpoints
 * A list of point, short-night will blocking when encounter that points.
 * @property {boolean} playAnimation
 * The short-night will unblock one breakpoint after some times when this been set true.
 * When this be set true and not specify breakpoints, the breakpoints will be
 * set as BreakpointAnimation.defaultAnimationBreakpoints to play animation.
 * @property {boolean} autoScroll - auto scroll to focus when playing animation.
 * @property {function} timeoutCounter - count a timeout to wait that point.
 * @property {boolean} scrollDuration - the duration of scroll.
 * @property {boolean} scrollContainer - scroll what. The default value is window.
 * */
export interface BreakpointAnimationConfig {
    breakpoints: Breakpoint[];
    playAnimation: boolean;
    autoScroll: boolean;
    timeoutCounter: (point: Breakpoint, config: Partial<BlockConfig>) => number;
    scrollDuration: number;
    scrollContainer: Window | HTMLElement;
}
/**
 * Config for this extension.
 * */
export interface BlockConfig {
    protagonist: Component;
    components: Component[];
    forward: boolean;
    onBlock: () => Promise<void>;
    onNext: () => Promise<void>;
}
/**
 * 1. Support that set a Breakpoint .
 * 2. Support that play animation base on Breakpoints.
 * */
export default class BreakpointAnimation implements Partial<Extension>, BreakpointAnimationConfig {
    ext: ExtensionManager;
    constructor(ext: ExtensionManager, { breakpoints, playAnimation, autoScroll, timeoutCounter, scrollDuration, scrollContainer, }?: Partial<BreakpointAnimationConfig>);
    breakpoints: BreakpointAnimationConfig['breakpoints'];
    readonly playAnimation: BreakpointAnimationConfig['playAnimation'];
    timeoutCounter: BreakpointAnimationConfig['timeoutCounter'];
    autoScroll: BreakpointAnimationConfig['autoScroll'];
    scrollDuration: BreakpointAnimationConfig['scrollDuration'];
    scrollContainer: BreakpointAnimationConfig['scrollContainer'];
    /**
     * Scroll to top when end of play animation.
     * */
    onDraw(comp: Component): void;
    /**
     * Blocking at a breakpoint util next called
     * */
    block(point: Breakpoint, config?: Partial<BlockConfig>): Promise<void>;
    /**
     * Step into next breakpoint.
     * You can call window.next() to do same thing when DEBUG is true.
     * */
    next(): void;
    /**
     * Fill when short-night is blocking. Step in a breakpoint.
     * */
    protected stepIn?: () => Promise<void>;
    /** Get a instance of moveto */
    protected get moveTo(): moveto;
    /**
     * Count a number that be used scroll to focus when play animation.
     * */
    protected countScrollTarget(config: Partial<BlockConfig>): number | null;
    /**
     * @see BreakpointAnimationConfig.playAnimation
     * */
    static defaultAnimationBreakpoints: Breakpoint[];
}
